@@ -8,7 +8,7 @@ GPLv2 or later
 """
 
 MODULE_NAME = "odeviced.kernel26"
-__version__ = "0.9.9.8"
+__version__ = "0.9.9.9"
 
 from helpers import DBUS_INTERFACE_PREFIX, DBUS_PATH_PREFIX, readFromFile, writeToFile, cleanObjectName
 from framework.config import config
@@ -18,6 +18,7 @@ from framework.patterns.null import Null
 import dbus.service
 import gobject
 import os, time, sys, fcntl
+import calendar
 
 import logging
 logger = logging.getLogger( MODULE_NAME )
@@ -186,6 +187,16 @@ class LED( dbus.service.Object ):
             writeToFile( "%s/delay_on" % self.node, str( abs( delay_on ) ) )
             writeToFile( "%s/delay_off" % self.node, str( abs( delay_off ) ) )
 
+    @dbus.service.method( DBUS_INTERFACE, "iii", "" )
+    def BlinkSeconds( self, seconds, delay_on, delay_off ):
+        # define stop function
+        def stopIt( self=self ):
+            self.SetBrightness( 0 )
+            return False # remove timer, don't call again
+        # start
+        self.SetBlinking( delay_on, delay_off )
+        gobject.timeout_add_seconds( seconds, stopIt )
+
     @dbus.service.method( DBUS_INTERFACE, "ss", "" )
     def SetNetworking( self, interface, mode ):
         # validate parameters
@@ -232,12 +243,9 @@ class PowerSupply( dbus.service.Object ):
             gobject.idle_add( self.onColdstart )
 
     def isPresent( self ):
-        present = readFromFile( "%s/present" % self.node )
-        online = readFromFile( "%s/online" % self.node )
-        return ( present == "1" or online == "1" )
-
-    def isOnline( self ):
-        return not ( readFromFile( "%s/online" % self.node ) != '1' )
+        toRead = "%s/present" if self.isBattery else "%s/online"
+        present = readFromFile( toRead % self.node )
+        return ( present == "1" )
 
     def theType( self ):
         return readFromFile( "%s/type" % self.node )
@@ -269,10 +277,12 @@ class PowerSupply( dbus.service.Object ):
 
     def onCapacityCheck( self ):
         if not self.isPresent():
+            logger.warning( "Power supply %s has been removed. On AC.", self )
+            self.sendPowerStatusIfChanged( "ac" )
             return True # call me again
         capacity = self.readCapacity()
         self.sendCapacityIfChanged( capacity )
-        if self.online:
+        if self.isPresent():
             if capacity > 98: # older batteries will never reach 100
                 self.sendPowerStatusIfChanged( "full" )
         else: # offline
@@ -404,22 +414,22 @@ class PowerSupplyApm( dbus.service.Object ):
         return int( percentage[:-1] )
 
 #----------------------------------------------------------------------------#
-class RealTimeClock( dbus.service.Object ):
+class RealtimeClock( dbus.service.Object ):
 #----------------------------------------------------------------------------#
-    """A Dbus Object implementing org.freesmartphone.Device.RealTimeClock
+    """A Dbus Object implementing org.freesmartphone.Device.RealtimeClock
     using the kernel 2.6 rtc class device"""
-    DBUS_INTERFACE = DBUS_INTERFACE_PREFIX + ".RealTimeClock"
+    DBUS_INTERFACE = DBUS_INTERFACE_PREFIX + ".RealtimeClock"
     SUPPORTS_MULTIPLE_OBJECT_PATHS = True
     OBJECT_PATH_COUNTER = 0
 
     def __init__( self, bus, index, node ):
         self.interface = self.DBUS_INTERFACE
-        self.path = DBUS_PATH_PREFIX + "/RealTimeClock/%s" % cleanObjectName( node.split('/')[-1] )
+        self.path = DBUS_PATH_PREFIX + "/RTC/%s" % cleanObjectName( node.split('/')[-1] )
         dbus.service.Object.__init__( self, bus, self.path )
         logger.info( "%s %s initialized. Serving %s at %s" % ( self.__class__.__name__, __version__, self.interface, self.path ) )
         self.node = node
         # also register object under an incremental path
-        self.path2 = DBUS_PATH_PREFIX + "/RealTimeClock/%d" % self.__class__.OBJECT_PATH_COUNTER
+        self.path2 = DBUS_PATH_PREFIX + "/RTC/%d" % self.__class__.OBJECT_PATH_COUNTER
         self.add_to_connection( self._connection, self.path2  )
         self.__class__.OBJECT_PATH_COUNTER += 1
 
@@ -430,39 +440,40 @@ class RealTimeClock( dbus.service.Object ):
     def GetName( self ):
         return self.node.split("/")[-1]
 
-    @dbus.service.method( DBUS_INTERFACE, "", "s" )
+    @dbus.service.method( DBUS_INTERFACE, "", "i" )
     def GetCurrentTime( self ):
         """Return seconds since epoch (UTC)"""
-        return readFromFile( "%s/since_epoch" % self.node )
+        return int( readFromFile( "%s/since_epoch" % self.node ) )
 
-    @dbus.service.method( DBUS_INTERFACE, "s", "" )
+    @dbus.service.method( DBUS_INTERFACE, "i", "" )
     def SetCurrentTime( self, t ):
         """Set time by seconds since epoch (UTC)"""
         pyrtc.rtcSetTime( time.gmtime( t ) )
 
-    @dbus.service.method( DBUS_INTERFACE, "", "s" )
+    @dbus.service.signal( DBUS_INTERFACE, "i" )
+    def WakeupTimeChanged ( self, t ):
+        pass
+
+    @dbus.service.method( DBUS_INTERFACE, "", "i" )
     def GetWakeupTime( self ):
         """Return wakeup time in seconds since epoch (UTC) if a wakeup
         time has been set. Return 0, otherwise."""
         # the wakealarm attribute is not always present
-        if os.path.exists( "%s/wakealarm" % self.node ):
-            return readFromFile( "%s/wakealarm" % self.node )
-        else:
-            # use ioctl interface
-            ( enabled, pending ), t = pyrtc.rtcReadAlarm()
-            return "0" if not enabled else str( time.mktime( t ) )
+        result = pyrtc.rtcReadAlarm()
+        if result is None:
+            logger.error( "Can't read RTC alarm. Returning 0" )
+            result = ( ( False, False ), 0 )
+        ( enabled, pending ), t = result
+        return 0 if not enabled else calendar.timegm( t )
 
-    @dbus.service.method( DBUS_INTERFACE, "s", "" )
+    @dbus.service.method( DBUS_INTERFACE, "i", "" )
     def SetWakeupTime( self, t ):
         """Set wakeup time in seconds since epoch (UTC). Set 0 to disable."""
-        if os.path.exists( "%s/wakealarm" % self.node ):
-            writeToFile( "%s/wakealarm" % self.node, t )
+        if t == 0:
+            pyrtc.rtcDisableAlarm()
         else:
-            # use ioctl interface
-            if t == "0":
-                pyrtc.rtcDisableAlarm()
-            else:
-                pyrtc.rtcSetAlarm( time.gmtime( float(t) ) )
+            pyrtc.rtcSetAlarm( time.gmtime( t ) )
+        self.WakeupTimeChanged( t )
 
 #----------------------------------------------------------------------------#
 def factory( prefix, controller ):
@@ -487,12 +498,7 @@ def factory( prefix, controller ):
             logger.debug( "scanning %s %s", index, node )
             objects.append( LED( bus, index, "%s/%s" % ( ledpath, node ) ) )
 
-    # scan for power supplies (apm first, then power supply [kernel 2.6.24++])
-    powerpath = "/proc/apm"
-    logger.debug( "scanning %s", powerpath )
-    if os.path.exists( powerpath ):
-        objects.append( PowerSupplyApm( bus, 0, powerpath ) )
-
+    # scan for power supplies (kernel 2.6.24++)
     powerpath = "/sys/class/power_supply"
     if os.path.exists( powerpath ):
         for ( index, node ) in enumerate( os.listdir( powerpath ) ):
@@ -504,7 +510,7 @@ def factory( prefix, controller ):
     if os.path.exists( rtcpath ):
         for ( index, node ) in enumerate( os.listdir( rtcpath ) ):
             logger.debug( "scanning %s %s", index, node )
-            objects.append( RealTimeClock( bus, index, "%s/%s" % ( rtcpath, node ) ) )
+            objects.append( RealtimeClock( bus, index, "%s/%s" % ( rtcpath, node ) ) )
 
     return objects
 
@@ -562,7 +568,7 @@ if __name__ == "__main__":
 
     rtc = []
     for i in count():
-        iface = requestInterfaceForObject( DBUS_INTERFACE_PREFIX, RealTimeClock.DBUS_INTERFACE, DBUS_PATH_PREFIX+"/RealTimeClock/%s" % i )
+        iface = requestInterfaceForObject( DBUS_INTERFACE_PREFIX, RealtimeClock.DBUS_INTERFACE, DBUS_PATH_PREFIX+"/RTC/%s" % i )
         if iface is not None:
             rtc.append( iface )
         else:
