@@ -13,7 +13,7 @@ Module: fso_actions
 
 """
 
-__VERSION__ = "0.4.3"
+__VERSION__ = "0.5.1.1"
 MODULE_NAME = "oeventsd"
 
 import framework.patterns.tasklet as tasklet
@@ -22,6 +22,7 @@ from action import Action
 from action import QueuedDBusAction, DBusAction
 from framework.controller import Controller
 from framework.config import installprefix
+from framework.patterns import dbuscache
 
 import gobject
 import dbus
@@ -40,13 +41,12 @@ class SetProfile( Action ):
 
     @tasklet.tasklet
     def __trigger( self ):
-        # We store the current profile
+        prefs = dbuscache.dbusInterfaceForObjectWithInterface(
+            "org.freesmartphone.opreferencesd",
+            "/org/freesmartphone/Preferences",
+            "org.freesmartphone.Preferences" )
+        # First, store the current profile
         # FIXME: we should use profile push and pop instead
-        prefs = dbus.SystemBus().get_object(
-            'org.freesmartphone.opreferencesd',
-            '/org/freesmartphone/Preferences'
-        )
-        prefs = dbus.Interface(prefs, 'org.freesmartphone.Preferences')
         self.backup_profile = yield tasklet.WaitDBus( prefs.GetProfile )
         # Then we can set the profile
         yield tasklet.WaitDBus( prefs.SetProfile, self.profile )
@@ -151,55 +151,169 @@ class VibratorAction(Action):
     A dbus action on the Openmoko Neo Vibrator device
     """
     function_name = 'Vibration'
-    # FIXME: device specific, needs to go away from here / made generic (parametric? just take the first?)
-    def __init__(self, target = 'neo1973_vibrator'):
+    # FIXME: ATM it just takes the first and runs at full power, should fix.
+    def __init__(self, target = 0, mode = "continuous"):
+        self.mode = mode
         self.target = target
+
     def trigger(self, **kargs):
-        DBusAction(dbus.SystemBus(),
-                    'org.freesmartphone.odeviced',
-                    '/org/freesmartphone/Device/LED/%s' % self.target,
-                    'org.freesmartphone.Device.LED',
-                    'SetBlinking', 300, 700).trigger()
+        if self.mode == "continuous":
+            DBusAction(dbus.SystemBus(),
+                        'org.freesmartphone.odeviced',
+                        '/org/freesmartphone/Device/Vibrator/%s' % self.target,
+                        'org.freesmartphone.Device.Vibrator',
+                        'VibratePattern', 999, 300, 700, 90).trigger()
+        elif self.mode == "oneshot":
+            DBusAction(dbus.SystemBus(),
+                        'org.freesmartphone.odeviced',
+                        '/org/freesmartphone/Device/Vibrator/%s' % self.target,
+                        'org.freesmartphone.Device.Vibrator',
+                        'Vibrate', 400, 90).trigger()
+        else:
+            logger.warning( "invalid vibration mode '%s', valid are 'continuous' or 'oneshot'" )
 
     def untrigger(self, **kargs):
         DBusAction(dbus.SystemBus(),
                     'org.freesmartphone.odeviced',
-                    '/org/freesmartphone/Device/LED/%s' % self.target,
-                    'org.freesmartphone.Device.LED',
-                    'SetBrightness', 0).trigger()
+                    '/org/freesmartphone/Device/Vibrator/%s' % self.target,
+                    'org.freesmartphone.Device.Vibrator',
+                    'Stop').trigger()
+
+#============================================================================#
+class BTHeadsetPlayingAction(Action):
+#============================================================================#
+    """
+    A dbus action on the Bluetooth Headset API
+    """
+    function_name = 'BTHeadsetPlaying'
+    def __init__(self):
+        pass
+    def trigger(self, **kargs):
+        DBusAction(dbus.SystemBus(),
+                    'org.freesmartphone.ophoned',
+                    '/org/freesmartphone/Phone',
+                    'org.freesmartphone.Phone',
+                    'SetBTHeadsetPlaying', True).trigger()
+
+    def untrigger(self, **kargs):
+        DBusAction(dbus.SystemBus(),
+                    'org.freesmartphone.ophoned',
+                    '/org/freesmartphone/Phone',
+                    'org.freesmartphone.Phone',
+                    'SetBTHeadsetPlaying', False).trigger()
+
+    def __repr__(self):
+        return "BTHeadsetPlaying()"
 
 #============================================================================#
 class OccupyResourceAction(Action):
 #============================================================================#
     function_name = 'OccupyResource'
 
-    def __init__(self, resource):
+    usageiface = None
+    pending = {}
+    held = {}
+
+    @classmethod
+    def onResourceAvailable( cls, name, state ):
+        if state:
+            cls.updatePending( name )
+        # FIXME handle vanishing resources...
+
+    @classmethod
+    def updatePending( cls, name ):
+        try:
+            counter = cls.pending[name]
+        except KeyError:
+            pass
+        else:
+            if counter > 0:
+                cls.usageiface.RequestResource( name,
+                        reply_handler = lambda cls=cls,name=name: cls.onResourceRequestReply( name ),
+                        error_handler = lambda cls=cls,name=name, e=None: cls.onResourceRequestError( name, e ) )
+            else:
+                cls.usageiface.ReleaseResource( name,
+                        reply_handler = lambda cls=cls,name=name: cls.onResourceReleaseReply( name ),
+                        error_handler = lambda cls=cls,name=name, e=None: cls.onResourceReleaseError( name, e ) )
+
+    @classmethod
+    def onResourceRequestReply( cls, name ):
+        logger.debug( "onResourceRequestReply: %s" % name )
+        amount = cls.pending[name]
+        if amount > 0:
+            del cls.pending[name]
+            cls.held[name] = amount
+
+    @classmethod
+    def onResourceRequestError( cls, name, e ):
+        logger.debug( "onResourceRequestError: %s: %s" % ( name, e ) )
+
+    @classmethod
+    def onResourceReleaseReply( cls, name ):
+        logger.debug( "onResourceReleaseReply: %s" % name )
+        del cls.pending[name]
+
+    @classmethod
+    def onResourceReleaseError( cls, name, e ):
+        logger.debug( "onResourceReleaseError: %s: %s" % ( name, e ) )
+
+    def __init__( self, resource ):
         self.resource = resource
 
-    def trigger(self, **kargs):
-        DBusAction(dbus.SystemBus(),
-                    'org.freesmartphone.ousaged',
-                    '/org/freesmartphone/Usage',
-                    'org.freesmartphone.Usage',
-                    'RequestResource', self.resource).trigger()
+        if self.usageiface is None:
+            self.__class__.usageiface = dbuscache.dbusInterfaceForObjectWithInterface( \
+                    "org.freesmartphone.ousaged",
+                    "/org/freesmartphone/Usage",
+                    "org.freesmartphone.Usage" )
+            self.__class__.usageiface.connect_to_signal( "ResourceAvailable", self.__class__.onResourceAvailable )
 
-    def untrigger(self, **kargs):
-        DBusAction(dbus.SystemBus(),
-                    'org.freesmartphone.ousaged',
-                    '/org/freesmartphone/Usage',
-                    'org.freesmartphone.Usage',
-                    'ReleaseResource', self.resource).trigger()
+    def __repr__( self ):
+        return "OccupyResource(%s)" % ( self.resource )
+
+    def trigger( self, **kargs ):
+        if self.resource in self.__class__.pending:
+            self.__class__.pending[self.resource] = self.__class__.pending[self.resource] + 1
+
+        elif self.resource in self.__class__.held:
+            self.__class__.held[self.resource] = self.__class__.held[self.resource] + 1
+
+        else:
+            self.__class__.pending[self.resource] = 1
+            self.__class__.updatePending( self.resource )
+
+    def untrigger( self, **kargs ):
+        if self.resource in self.__class__.pending:
+            counter = self.__class__.pending[self.resource]
+            if counter > 1:
+                self.__class__.pending[self.resource] = counter - 1
+            else:
+                self.__class__.pending[self.resource] = 0
+                self.__class__.updatePending( self.resource )
+
+        elif self.resource in self.__class__.held:
+            counter = self.__class__.held[self.resource]
+            if counter > 1:
+                self.__class__.held[self.resource] = counter - 1
+            else:
+                del self.__class__.held[self.resource]
+                self.__class__.pending[self.resource] = 0
+                self.__class__.updatePending( self.resource )
+
+        else:
+            logger.error( "Untrigger for resource %s before trigger!? Ignoring..." % self.resource )
 
 #=========================================================================#
-class RingToneAction(Action):
+class UserAlertAction(Action):
 #=========================================================================#
-    function_name = 'RingTone'
+    function_name = '___abstract_dont_use_this_directly____'
 
     def __init__( self, *args, **kwargs ):
         Action.__init__( self )
-        logger.debug( "%s: init" )
+        logger.debug( "%s: init", self )
         self.audio_action = None
         self.vibrator_action = None
+        self.eventname = self.__class__.event_name # needs to be populated in derived classes
+        self.vibratormode = self.__class__.vibrator_mode # dito
         gobject.idle_add( self.initFromMainloop )
 
     def initFromMainloop( self ):
@@ -211,117 +325,106 @@ class RingToneAction(Action):
         Audio preferences have changed. Reload.
         """
         k = str(key)
-        if k == "ring-tone":
-            self.ring_tone = value
-        elif k == "ring-volume":
-            self.ring_volume = value
-        elif k == "ring-loop":
-            self.ring_loop = value
-        elif k == "ring-length":
-            self.ring_length = value
+        if k == "%s-tone" % self.eventname:
+            self.tone = str(value)
+        elif k == "%s-volume" % self.eventname:
+            self.volume = int(value)
+        elif k == "%s-loop" % self.eventname:
+            self.loop = int(value)
+        elif k == "%s-length" % self.eventname:
+            self.length = int(value)
+        elif k == "%s-vibration" % self.eventname:
+            self.vibrate = int(value)
 
-        self.sound_path = os.path.join( installprefix, "share/sounds/", self.ring_tone )
-        self.audio_action = AudioAction(self.sound_path, self.ring_loop, self.ring_length) if self.ring_volume != 0 else None
-        self.vibrator_action = VibratorAction()
+        self.sound_path = os.path.join( installprefix, "share/sounds/", self.tone )
+        self.audio_action = AudioAction( self.sound_path, self.loop, self.length ) if self.volume != 0 else None
+        self.vibrator_action = VibratorAction( mode=self.vibratormode ) if self.vibrate != 0 else None
 
-        logger.debug( "ring tone action changed: audio=%s, vibrator=%s", self.audio_action, self.vibrator_action )
+        logger.debug( "user alert action changed for %s: audio=%s, vibrator=%s", self.eventname, self.audio_action, self.vibrator_action )
 
     @tasklet.tasklet
-    def __init(self):
+    def __init( self ):
         """
         We need to make DBus calls and wait for the result,
         So we use a tasklet to avoid blocking the mainloop.
         """
 
-        logger.debug( "ring tone action init from mainloop" )
+        logger.debug( "user alert action init from mainloop for %s", self.eventname )
         # We get the 'phone' preferences service and
-        # retreive the ring-tone and ring-volume config values
+        # retreive the tone and volume config values
         # We are careful to use 'yield' cause the calls could be blocking.
         try:
-            prefs = dbus.SystemBus().get_object( "org.freesmartphone.opreferencesd", "/org/freesmartphone/Preferences" )
-            prefs = dbus.Interface( prefs, "org.freesmartphone.Preferences" )
-        except dbus.DBusException: # preferences daemon probably not present
-            logger.warning( "org.freesmartphone.opreferencesd not present. Can't get ring tones." )
-        else:
+            prefs = dbuscache.dbusInterfaceForObjectWithInterface(
+                "org.freesmartphone.opreferencesd",
+                "/org/freesmartphone/Preferences",
+                "org.freesmartphone.Preferences" )
             phone_prefs = yield tasklet.WaitDBus( prefs.GetService, "phone" )
-            phone_prefs = dbus.SystemBus().get_object( "org.freesmartphone.opreferencesd", phone_prefs )
-            phone_prefs = dbus.Interface( phone_prefs, "org.freesmartphone.Preferences.Service" )
+            phone_prefs = dbuscache.dbusInterfaceForObjectWithInterface(
+                "org.freesmartphone.opreferencesd",
+                phone_prefs,
+                "org.freesmartphone.Preferences.Service" )
 
+        except dbus.DBusException: # preferences daemon probably not present
+            logger.warning( "org.freesmartphone.opreferencesd not present. Can't get alert tones." )
+        else:
             # connect to signal for later notifications
             phone_prefs.connect_to_signal( "Notify", self.cbPreferencesServiceNotify )
 
             # FIXME does that still work if (some of) the entries are missing?
-            self.ring_tone = yield tasklet.WaitDBus( phone_prefs.GetValue, "ring-tone" )
-            self.ring_volume = yield tasklet.WaitDBus( phone_prefs.GetValue, "ring-volume" )
-            self.ring_loop = yield tasklet.WaitDBus( phone_prefs.GetValue, "ring-loop" )
-            self.ring_length = yield tasklet.WaitDBus( phone_prefs.GetValue, "ring-length" )
+            self.tone =    yield tasklet.WaitDBus( phone_prefs.GetValue, "%s-tone" % self.eventname )
+            self.volume =  yield tasklet.WaitDBus( phone_prefs.GetValue, "%s-volume" % self.eventname )
+            self.loop =    yield tasklet.WaitDBus( phone_prefs.GetValue, "%s-loop" % self.eventname )
+            self.length =  yield tasklet.WaitDBus( phone_prefs.GetValue, "%s-length" % self.eventname )
+            self.vibrate = yield tasklet.WaitDBus( phone_prefs.GetValue, "%s-vibration" % self.eventname )
 
-            self.sound_path = os.path.join( installprefix, "share/sounds/", self.ring_tone )
-            self.audio_action = AudioAction(self.sound_path, self.ring_loop, self.ring_length) if self.ring_volume != 0 else None
-            self.vibrator_action = VibratorAction()
+            self.tone = str(self.tone)
+            self.volume = int(self.volume)
+            self.loop = int(self.loop)
+            self.length = int(self.length)
+            self.vibrate = int(self.vibrate)
 
-        logger.debug( "ring tone action: audio=%s, vibrator=%s", self.audio_action, self.vibrator_action )
+            self.sound_path = os.path.join( installprefix, "share/sounds/", self.tone )
+            self.audio_action = AudioAction( self.sound_path, self.loop, self.length ) if self.volume != 0 else None
+            self.vibrator_action = VibratorAction( mode=self.vibratormode ) if self.vibrate != 0 else None
+
+        logger.debug( "user alert action for %s: audio=%s, vibrator=%s", self.eventname, self.audio_action, self.vibrator_action )
 
     def trigger(self, **kargs):
-        logger.info( "RingToneAction play" )
-        if self.audio_action:
+        logger.info( "UserAlertAction %s play", self.eventname )
+        if self.audio_action is not None:
             self.audio_action.trigger()
-        self.vibrator_action.trigger()
+        if self.vibrator_action is not None:
+            self.vibrator_action.trigger()
 
     def untrigger(self, **kargs):
-        logger.info( "RingToneAction stop" )
-        if self.audio_action:
+        logger.info( "UserAlertAction %s stop", self.eventname )
+        if self.audio_action is not None:
             self.audio_action.untrigger()
-        if self.vibrator_action:
+        if self.vibrator_action is not None:
             self.vibrator_action.untrigger()
 
+    def __repr__(self):
+        return "Abstract UserAlertAction()"
+
+#=========================================================================#
+class RingToneAction(UserAlertAction):
+#=========================================================================#
+    function_name = 'RingTone'
+    event_name = "ring"
+    vibrator_mode = "continuous"
+
     def __repr__(self):
         return "RingToneAction()"
 
 #=========================================================================#
-class MessageToneAction(Action):
+class MessageToneAction(UserAlertAction):
 #=========================================================================#
     function_name = 'MessageTone'
-
-    def __init__( self, cmd = 'play' ):
-        self.cmd = cmd
-
-    def trigger(self, **kargs):
-        logger.info( "MessageToneAction %s", self.cmd )
-
-        # We use the global Controller class to directly get the object
-        prefs = Controller.object( "/org/freesmartphone/Preferences" )
-        if prefs is None:
-            logger.error( "preferences not available and no default values defined." )
-            return
-        phone_prefs = prefs.GetService( "phone" )
-        tone = phone_prefs.GetValue( "message-tone" )
-        volume = phone_prefs.GetValue( "message-volume" )
-        loop = phone_prefs.GetValue( "message-loop" )
-        length = phone_prefs.GetValue( "message-length" )
-        sound_path = os.path.join( installprefix, "share/sounds/", tone )
-
-        # XXX: We don't set the ringing volume.
-        #      Here we only disable the audio action if the volume is 0
-        self.audio_action = AudioAction(sound_path, loop, length) if volume != 0 else None
-        #self.vibrator_action = VibratorAction()
-
-        if self.cmd == "play":
-            logger.info( "Start message tone : tone=%s, volume=%s, loop=%d, length=%d", tone, volume, loop, length )
-            if self.audio_action:
-                self.audio_action.trigger()
-            #self.vibrator_action.trigger()
-
-        elif self.cmd == "stop":
-            logger.info( "Stop message tone : tone=%s, volume=%s, loop=%d, length=%d", tone, volume, loop, length )
-            if self.audio_action: self.audio_action.untrigger()
-            if self.vibrator_action : self.vibrator_action.untrigger()
-        else:
-            logger.error( "Unknown MessageToneAction!" )
-            assert False, "unknown message tone action"
+    event_name = "message"
+    vibrator_mode = "oneshot"
 
     def __repr__(self):
-        return "MessageToneAction(%s)" % self.cmd
+        return "MessageToneAction()"
 
 #=========================================================================#
 class CommandAction(Action):
@@ -340,9 +443,9 @@ class CommandAction(Action):
         env.update( self.env )
 
         # FIXME if we are interested in tracking the process, then we
-        # should use glib's spawn async and add a chile watch
+        # should use glib's spawn async and add a child watch
         try:
-            subprocess.Popen( shlex.split( self.cmd ), env = env )
+            subprocess.call( shlex.split( self.cmd ), env = env )
         except Exception, e:
             logger.error( "Error while executing external command '%s': %s", self.cmd, e )
 
@@ -364,6 +467,21 @@ class SuspendAction(DBusAction):
         interface = 'org.freesmartphone.Usage'
         super(SuspendAction, self).__init__(bus, service, obj, interface, 'Suspend')
 
+#============================================================================#
+class ShutdownAction(DBusAction):
+#============================================================================#
+    """
+    A dbus action to shutdown the device
+    """
+    function_name = 'Shutdown'
+
+    def __init__(self):
+        bus = dbus.SystemBus()
+        service = 'org.freesmartphone.ousaged'
+        obj = '/org/freesmartphone/Usage'
+        interface = 'org.freesmartphone.Usage'
+        super(ShutdownAction, self).__init__(bus, service, obj, interface, 'Shutdown')
+
 #============================================================================#
 class ExternalDBusAction(DBusAction):
 #============================================================================#
@@ -408,4 +526,4 @@ class ExternalDBusAction(DBusAction):
 #============================================================================#
 if __name__ == "__main__":
 #============================================================================#
-    pass
\ No newline at end of file
+    pass
