@@ -16,6 +16,7 @@ import struct
 import calendar
 import dbus
 import dbus.service
+from framework.config import config
 
 import logging
 logger = logging.getLogger('ogpsd')
@@ -165,7 +166,7 @@ MSGFMT = {
     ("RXM-EPH", 104) :
         ["<" + "I"*26, ["SVID", "HOW", "SF1D0", "SF1D1", "SF1D2", "SF1D3", "SF1D4",
             "SF1D5", "SF1D6", "SF1D7", "SF2D0", "SF2D1", "SF2D2", "SF2D3", "SF2D4",
-            "SF2D5", "SF2D6", "SF1D7", "SF3D0", "SF3D1", "SF3D2", "SF3D3", "SF3D4", "SF3D5", "SF3D6", "SF3D7"]],
+            "SF2D5", "SF2D6", "SF2D7", "SF3D0", "SF3D1", "SF3D2", "SF3D3", "SF3D4", "SF3D5", "SF3D6", "SF3D7"]],
     ("INF-ERROR", None) :
         [0, "", [], 1, "c", ["Char"]],
     ("INF-WARNING", None) :
@@ -255,7 +256,7 @@ MSGFMT = {
     ("AID-EPH", 104) :
         ["<" + "I"*26, ["SVID", "HOW", "SF1D0", "SF1D1", "SF1D2", "SF1D3", "SF1D4",
             "SF1D5", "SF1D6", "SF1D7", "SF2D0", "SF2D1", "SF2D2", "SF2D3", "SF2D4",
-            "SF2D5", "SF2D6", "SF1D7", "SF3D0", "SF3D1", "SF3D2", "SF3D3", "SF3D4", "SF3D5", "SF3D6", "SF3D7"]]
+            "SF2D5", "SF2D6", "SF2D7", "SF3D0", "SF3D1", "SF3D2", "SF3D3", "SF3D4", "SF3D5", "SF3D6", "SF3D7"]]
 # TIM - Timekeeping
 }
 
@@ -276,8 +277,26 @@ class UBXDevice( GPSDevice ):
     def initializeDevice( self ):
         # Use high sensitivity mode
         #self.send("CFG-RXM", 2, {"gps_mode" : 2, "lp_mode" : 0})
-        # Enable use of SBAS (even in testmode)
+        # Enable use of SBAS
         self.send("CFG-SBAS", 8, {"mode" : 1, "usage" : 7, "maxsbas" : 3, "scanmode" : 0})
+        # Setup NAV2 parameters (all parameters other than StaticThres are the same as the default
+        self.send("CFG-NAV2", 40, {"DGPSTO": 60,
+                "FixMode": 2,
+                "FixedAltitude": 50000,
+                "MaxDR": 5,
+                "MaxSVs": 16,
+                "MinCN0After": 10,
+                "MinCN0Initial": 15,
+                "MinELE": 5,
+                "MinSVInitial": 3,
+                "MinSVs": 3,
+                "NAVOPT": 1,
+                "PACC": 100,
+                "PDOP": 250,
+                "Platform": 3, # Automotive = 3, pedestrian = 2, stationary = 1
+                "StaticThres": config.getInt( "ogpsd", "static_threshold", 10),
+                "TACC": 300,
+                "TDOP": 250})
 
         # Disable NMEA for current port
         self.ubx["CFG-PRT"] = {"In_proto_mask" : 1, "Out_proto_mask" : 1}
