@@ -11,7 +11,7 @@ Module: device
 """
 
 MODULE_NAME = "ogsmd.device"
-__version__ = "0.9.11.0"
+__version__ = "0.9.14"
 
 from framework import resource
 from framework.config import config
@@ -81,6 +81,11 @@ class Device( resource.Resource ):
             on_error( resource.ResourceError( estring ) )
         else:
             self.modem = Modem( self, self.bus )
+            if self.modem is None:
+                estring = "Cannot create Modem %s" % ( self.modemtype )
+                logger.error( estring )
+                on_error( resource.ResourceError( estring ) )
+                return
             setCurrentModem( self.modem )
             self.modem.open( on_ok, on_error )
 
@@ -98,13 +103,22 @@ class Device( resource.Resource ):
         """
         Suspend (inherited from Resource)
         """
-        self.modem.prepareForSuspend( on_ok, on_error )
+        if self.modem is not None:
+            self.modem.prepareForSuspend( on_ok, on_error )
 
     def _resume( self, on_ok, on_error ):
         """
         Resume (inherited from Resource)
         """
-        self.modem.recoverFromSuspend( on_ok, on_error )
+        if self.modem is not None:
+            self.modem.recoverFromSuspend( lambda: self._recoverOk( on_ok, on_error ), on_error )
+
+    def _recoverOk( self, on_ok, on_error):
+        mediator.NetworkGetStatus( self, lambda x: self._recoverStatusOk( x, on_ok, on_error ), on_error )
+
+    def _recoverStatusOk( self, status, on_ok, on_error ):
+        self.Status( status )
+        on_ok()
 
     #
     # dbus org.freesmartphone.GSM.Device
@@ -175,6 +189,29 @@ class Device( resource.Resource ):
     def SetMicrophoneMuted( self, muted, dbus_ok, dbus_error ):
         mediator.DeviceSetMicrophoneMuted( self, dbus_ok, dbus_error, muted=muted )
 
+    @dbus.service.method( DBUS_INTERFACE_DEVICE, "", "si",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def GetPowerStatus( self, dbus_ok, dbus_error ):
+        mediator.DeviceGetPowerStatus( self, dbus_ok, dbus_error )
+
+    @dbus.service.method( DBUS_INTERFACE_DEVICE, "", "",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def SetRTC( self, dbus_ok, dbus_error ):
+        mediator.DeviceSetRTC( self, dbus_ok, dbus_error )
+
+    @dbus.service.method( DBUS_INTERFACE_DEVICE, "", "i",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def GetRTC( self, dbus_ok, dbus_error ):
+        mediator.DeviceGetRTC( self, dbus_ok, dbus_error )
+
+    @resource.queuedsignal
+    @dbus.service.signal( DBUS_INTERFACE_DEVICE, "sb" )
+    def KeypadEvent( self, name, pressed ):
+        logger.info( "key %s has been %s", name, "pressed" if pressed else "released" )
+
     #
     # dbus org.freesmartphone.GSM.SIM
     #
@@ -264,7 +301,7 @@ class Device( resource.Resource ):
     def GetIssuer( self, dbus_ok, dbus_error ):
         mediator.SimGetIssuer( self, dbus_ok, dbus_error )
 
-    @dbus.service.method( DBUS_INTERFACE_SIM, "", "a{is}",
+    @dbus.service.method( DBUS_INTERFACE_SIM, "", "a{ss}",
                           async_callbacks=( "dbus_ok", "dbus_error" ) )
     @resource.checkedmethod
     def GetProviderList( self, dbus_ok, dbus_error ):
@@ -283,11 +320,17 @@ class Device( resource.Resource ):
     def GetPhonebookInfo( self, category, dbus_ok, dbus_error ):
         mediator.SimGetPhonebookInfo( self, dbus_ok, dbus_error, category=category )
 
-    @dbus.service.method( DBUS_INTERFACE_SIM, "s", "a(iss)",
+    @dbus.service.method( DBUS_INTERFACE_SIM, "s", "ii",
                           async_callbacks=( "dbus_ok", "dbus_error" ) )
     @resource.checkedmethod
-    def RetrievePhonebook( self, category, dbus_ok, dbus_error ):
-        mediator.SimRetrievePhonebook( self, dbus_ok, dbus_error, category=category )
+    def GetPhonebookStorageInfo( self, category, dbus_ok, dbus_error ):
+        mediator.SimGetPhonebookStorageInfo( self, dbus_ok, dbus_error, category=category )
+
+    @dbus.service.method( DBUS_INTERFACE_SIM, "sii", "a(iss)",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def RetrievePhonebook( self, category, indexFirst, indexLast, dbus_ok, dbus_error ):
+        mediator.SimRetrievePhonebook( self, dbus_ok, dbus_error, category=category, indexFirst=indexFirst, indexLast=indexLast )
 
     @dbus.service.method( DBUS_INTERFACE_SIM, "si", "",
                           async_callbacks=( "dbus_ok", "dbus_error" ) )
@@ -375,11 +418,28 @@ class Device( resource.Resource ):
     def SendMessage( self, number, contents, properties, dbus_ok, dbus_error ):
         mediator.SmsSendMessage( self, dbus_ok, dbus_error, number=number, contents=contents, properties=properties )
 
+    @dbus.service.method( DBUS_INTERFACE_SMS, "sa{sv}", "",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def AckMessage( self, contents, properties, dbus_ok, dbus_error ):
+        mediator.SmsAckMessage( self, dbus_ok, dbus_error, contents=contents, properties=properties )
+
+    @dbus.service.method( DBUS_INTERFACE_SMS, "sa{sv}", "",
+                          async_callbacks=( "dbus_ok", "dbus_error" ) )
+    @resource.checkedmethod
+    def NackMessage( self, contents, properties, dbus_ok, dbus_error ):
+        mediator.SmsNackMessage( self, dbus_ok, dbus_error, contents=contents, properties=properties )
+
     @resource.queuedsignal
     @dbus.service.signal( DBUS_INTERFACE_SMS, "ssa{sv}" )
-    def IncomingMessage( self, address, text, features ):
+    def IncomingMessage( self, address, text, properties ):
         logger.info( "incoming message (unbuffered) from %s", address )
 
+    @resource.queuedsignal
+    @dbus.service.signal( DBUS_INTERFACE_SMS, "ssa{sv}" )
+    def IncomingMessageReceipt( self, number, text, properties ):
+        logger.info( "incoming message delivery report from %s", number )
+
     #
     # dbus org.freesmartphone.GSM.Network
     #
@@ -417,13 +477,13 @@ class Device( resource.Resource ):
     def SignalStrength( self, strength ):
         logger.info( "org.freesmartphone.GSM.Network.SignalStrength: %s", strength )
 
-    @dbus.service.method( DBUS_INTERFACE_NETWORK, "", "a(issss)",
+    @dbus.service.method( DBUS_INTERFACE_NETWORK, "", "a(sssss)",
                           async_callbacks=( "dbus_ok", "dbus_error" ) )
     @resource.checkedmethod
     def ListProviders( self, dbus_ok, dbus_error ):
         mediator.NetworkListProviders( self, dbus_ok, dbus_error )
 
-    @dbus.service.method( DBUS_INTERFACE_NETWORK, "i", "",
+    @dbus.service.method( DBUS_INTERFACE_NETWORK, "s", "",
                           async_callbacks=( "dbus_ok", "dbus_error" ) )
     @resource.checkedmethod
     def RegisterWithProvider( self, operator_code, dbus_ok, dbus_error ):
