@@ -3,8 +3,8 @@
 The Open GSM Daemon - Python Implementation
 
 (C) 2007-2008 M. Dietrich.
-(C) 2008 Michael 'Mickey' Lauer <mlauer@vanille-media.de>
-(C) 2008 Openmoko, Inc.
+(C) 2008-2009 Michael 'Mickey' Lauer <mlauer@vanille-media.de>
+(C) 2008-2009 Openmoko, Inc.
 GPLv2 or later
 
 Package: ogsmd.gsm
@@ -14,20 +14,30 @@ This module provides communication channel abstractions that
 transport their data over a (virtual) serial line.
 """
 
-__version__ = "0.9.9.5"
+__version__ = "0.9.18.2"
 MODULE_NAME = "ogsmd.channel"
 
-from ogsmd.gsm.decor import logged
 import parser
 
 import gobject # pygobject
 
 import serial # pyserial
-import Queue, fcntl, os, time, types # stdlib
+import Queue, fcntl, os, time, types, re # stdlib
 
 import logging
 logger = logging.getLogger( MODULE_NAME )
 
+PRIORITY_RTR = -20
+PRIORITY_RTS = -10
+PRIORITY_HUP = -5
+
+DEFAULT_CHANNEL_TIMEOUT = 5*60
+
+AUTOPREFIX = re.compile( "A?T?(?P<prefix>[\+%!*$\^_&@][A-Z]+)" )
+
+# keys are commands, values are autocomputed response lists
+AUTOPREFIX_CACHE = {}
+
 #=========================================================================#
 class PeekholeQueue( Queue.Queue ):
 #=========================================================================#
@@ -51,7 +61,6 @@ class VirtualChannel( object ):
     #
     # public API
     #
-    @logged
     def __init__( self, pathfactory, name=None, **kwargs ):
         """Construct"""
         self.pathfactory = pathfactory
@@ -64,13 +73,13 @@ class VirtualChannel( object ):
     def __repr__( self ):
         return "<%s via %s>" % ( self.__class__.__name__, self.serial.port if self.serial is not None else "unknown" )
 
-    @logged
     def open( self, path=None ):
         """
         Allocate a channel and opens a serial port.
         Returns True, if successful. False, otherwise.
         """
-        assert not self.connected, "already connected"
+        if self.connected:
+            raise ValueError( "already connected" )
 
         # gather path
         path = self.pathfactory( self.name )
@@ -103,9 +112,9 @@ class VirtualChannel( object ):
             return False
 
         # set up I/O watches for mainloop
-        self.watchReadyToRead = gobject.io_add_watch( self.serial.fd, gobject.IO_IN, self._readyToRead )
-        self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend )
-        self.watchHUP = gobject.io_add_watch( self.serial.fd, gobject.IO_HUP, self._hup )
+        self.watchReadyToRead = gobject.io_add_watch( self.serial.fd, gobject.IO_IN, self._readyToRead, priority=PRIORITY_RTR )
+        self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend, priority=PRIORITY_RTS )
+        self.watchHUP = gobject.io_add_watch( self.serial.fd, gobject.IO_HUP, self._hup, priority=PRIORITY_HUP )
         self.connected = self.serial.isOpen()
         return self.connected
 
@@ -128,7 +137,6 @@ class VirtualChannel( object ):
         """
         pass
 
-    @logged
     def suspend( self, ok_callback, error_callback ):
         """
         Called when the channel needs to be prepared for a suspend.
@@ -137,7 +145,6 @@ class VirtualChannel( object ):
         """
         ok_callback( self )
 
-    @logged
     def resume( self, ok_callback, error_callback ):
         """
         Called when the channel needs to reinit after resume.
@@ -146,12 +153,24 @@ class VirtualChannel( object ):
         """
         ok_callback( self )
 
-    @logged
     def write( self, data ):
-        """Write data to the modem."""
+        """Write data to the transport."""
         self._write( data )
 
-    @logged
+    def freeze( self ):
+        """Pause reading from the transport."""
+        if not self.watchReadyToRead:
+            logger.warning( "%s: freeze() called without watch being setup", self )
+        else:
+            gobject.source_remove( self.watchReadyToRead )
+
+    def thaw( self ):
+        """Resume reading from the transport."""
+        if not self.watchReadyToRead:
+            logger.warning( "%s: thaw() called with watch being already setup", self )
+        else:
+            self.watchReadyToRead = gobject.io_add_watch( self.serial.fd, gobject.IO_IN, self._readyToRead, priority=PRIORITY_RTR )
+
     def close( self ):
         """
         Close the serial port and free the virtual channel.
@@ -170,6 +189,12 @@ class VirtualChannel( object ):
         self.connected = self.serial.isOpen()
         return not self.connected
 
+    def port( self ):
+        """
+        Return name of transport.
+        """
+        return self.serial.port
+
     #
     # hooks
     #
@@ -200,11 +225,11 @@ class VirtualChannel( object ):
     #
     # private API
     #
-    @logged
     def _hup( self, source, condition ):
         """Called, if there is a HUP condition on the source."""
-        assert source == self.serial.fd, "HUP on bogus source"
-        assert condition == gobject.IO_HUP, "HUP on bogus condition"
+        if ( source != self.serial.fd or condition != gobject.IO_HUP ):
+            logger.warning( "ready to read, but bogus condition %d or source %d. Ignoring", condition, source )
+            return False
         logger.info( "%s: HUP on socket" % self )
 
         self._hookHandleHupCondition()
@@ -213,13 +238,14 @@ class VirtualChannel( object ):
 
     def _readyToRead( self, source, condition ):
         """Called, if data is available on the source."""
-        assert source == self.serial.fd, "ready to read on bogus source"
-        assert condition == gobject.IO_IN, "ready to read on bogus condition"
-        logger.debug( "%s: _readyToRead: watch timeout = %s", self, repr( self.watchTimeout ) )
+        if ( source != self.serial.fd or condition != gobject.IO_IN ):
+            logger.warning( "ready to read, but bogus condition %d or source %d. Ignoring", condition, source )
+            return False
+        #logger.debug( "%s: _readyToRead: watch timeout = %s", self, repr( self.watchTimeout ) )
 
         self._hookPreReading()
         data = self._lowlevelRead()
-        logger.debug( "%s: got %d bytes from: %s" % ( self, len(data), repr(data) ) )
+        logger.debug( "%s: got %d bytes: %s" % ( self, len(data), repr(data) ) )
         self.readyToRead( data )
 
         self._hookPostReading()
@@ -240,9 +266,16 @@ class VirtualChannel( object ):
 
     def _readyToSend( self, source, condition ):
         """Called, if source is ready to receive data."""
-        assert source == self.serial.fd, "ready to write on bogus source"
-        assert condition == gobject.IO_OUT, "ready to write on bogus condition"
-        logger.debug( "%s: _readyToSend: watch timeout = %s", self, repr( self.watchTimeout ) )
+        if ( source != self.serial.fd or condition != gobject.IO_OUT ):
+            logger.warning( "ready to send, but bogus condition %d or source %d. Ignoring", condition, source )
+            return False
+        #logger.debug( "%s: _readyToSend: watch timeout = %s", self, repr( self.watchTimeout ) )
+
+        if False:
+            # make sure nothing has been queued up in the buffer in the meantime
+            while self.serial.inWaiting():
+                logger.warning( "_readyToSend, but new data already in the buffer. processing" )
+                self._readyToRead( self.serial.fd, gobject.IO_IN )
 
         self._hookPreSending()
         self.readyToSend()
@@ -251,7 +284,6 @@ class VirtualChannel( object ):
 
         return False # gobject, don't call me again
 
-    @logged
     def __del__( self ):
         """Destruct"""
         self.close()
@@ -279,10 +311,8 @@ class QueuedVirtualChannel( VirtualChannel ):
         self.installParser()
 
         self.watchTimeout = None
-        if "timeout" in kwargs:
-            self.timeout = kwargs["timeout"]
-        else:
-            self.timeout = 10 # default timeout in seconds
+
+        self.timeout = kwargs.get( "timeout", DEFAULT_CHANNEL_TIMEOUT )
 
         logger.info( "%s: Creating channel with timeout = %d seconds", self, self.timeout )
 
@@ -294,17 +324,25 @@ class QueuedVirtualChannel( VirtualChannel ):
         """
         self.parser = parser.LowlevelAtParser( self._handleResponseToRequest, self._handleUnsolicitedResponse )
 
-    def enqueue( self, data, response_cb=None, error_cb=None, timeout=None ):
+    def enqueue( self, data, response_cb=None, error_cb=None, prefixes=None ):
         """
         Enqueue data block for sending over the channel.
         """
+
+        if prefixes is None:
+            try:
+                prefixes = AUTOPREFIX_CACHE[data]
+            except KeyError:
+                AUTOPREFIX_CACHE[data] = prefixes = set( AUTOPREFIX.findall( data ) )
+                logger.debug( "%s: Autogenerated prefixes for command %s: %s", self, repr(data), prefixes )
+
         if type( data ) == types.UnicodeType:
             data = str( data )
-        self.q.put( ( data, response_cb, error_cb, timeout or self.timeout ) )
+        self.q.put( ( data, response_cb, error_cb, prefixes ) )
         if not self.connected:
             return
         if self.q.qsize() == 1 and not self.watchReadyToSend:
-            self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend )
+            self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend, priority=PRIORITY_RTS )
 
     def pendingCommands( self ):
         """
@@ -319,6 +357,15 @@ class QueuedVirtualChannel( VirtualChannel ):
         """
         return self.watchTimeout is not None
 
+    def validPrefixes( self ):
+        """
+        Return a list of prefixes that are valid for the command in execution.
+        """
+        if not self.isWaitingForResponse():
+            return []
+
+        return self.q.peek()[3]
+
     def cancelCurrentCommand( self ):
         """
         Cancel the command currently in process.
@@ -327,7 +374,6 @@ class QueuedVirtualChannel( VirtualChannel ):
             return
         self._handleCommandCancellation()
 
-    #@logged
     def readyToSend( self ):
         """
         Reimplemented for internal purposes.
@@ -338,17 +384,19 @@ class QueuedVirtualChannel( VirtualChannel ):
 
         logger.debug( "%s: sending %d bytes: %s" % ( repr(self), len(self.q.peek()[0]), repr(self.q.peek()[0]) ) )
         self._lowlevelWrite( self.q.peek()[0] ) # 0 = request data
-        if self.q.peek()[3]: # 3 = request timeout
-            self.watchTimeout = gobject.timeout_add_seconds( self.q.peek()[3], self._handleCommandTimeout )
-        else:
-            self.watchTimeout = gobject.timeout_add_seconds( self.timeout, self._handleCommandTimeout )
+        self.watchTimeout = gobject.timeout_add_seconds( self.timeout, self._handleCommandTimeout )
         return False
 
     def readyToRead( self, data ):
         """
         Reimplemented for internal purposes.
         """
-        self.parser.feed( data, self.isWaitingForResponse() )
+
+        # restart timeout //FIXME: only if we were waiting for a response?
+        if self.watchTimeout is not None:
+            gobject.source_remove( self.watchTimeout )
+            self.watchTimeout = gobject.timeout_add_seconds( self.timeout, self._handleCommandTimeout )
+        self.parser.feed( data, self.isWaitingForResponse(), self.validPrefixes() )
 
     def handleUnsolicitedResponse( self, response ):
         """
@@ -379,7 +427,7 @@ class QueuedVirtualChannel( VirtualChannel ):
                 else:
                     ok_cb( reqstring.strip(), response )
             except Exception, e:
-                logger.exception( "unhandled exception in response callback: %s" % e )
+                logger.exception( "(ignoring) unhandled exception in response callback: %s" % e )
 
     def handleCommandTimeout( self, request ):
         """
@@ -387,18 +435,16 @@ class QueuedVirtualChannel( VirtualChannel ):
 
         The default implementation calls the error callback pinned to the request.
         """
-        reqstring, ok_cb, error_cb, timeout = request
+        reqstring, ok_cb, error_cb, prefixes = request
         if not ok_cb and not error_cb:
             logger.debug( "%s: TIMEOUT '%s' => ???" % ( repr(self), reqstring.strip() ) )
         else:
             logger.debug( "%s: TIMEOUT '%s' => ???" % ( repr(self), reqstring.strip() ) )
-            error_cb( reqstring.strip(), ( "timeout", timeout ) )
+            error_cb( reqstring.strip(), ( "timeout", self.timeout ) )
 
     #
     # private API
     #
-
-    #@logged
     def _handleCommandCancellation( self ):
         """
         Called, when the current command should be cancelled.
@@ -406,7 +452,9 @@ class QueuedVirtualChannel( VirtualChannel ):
         According to v25ter, this can be done by sending _any_
         character to the serial line.
         """
-        assert self.watchTimeout is not None, "no command to cancel"
+        if self.watchTimeout is None:
+            logger.warning( "no command to cancel" )
+            return
         # we have a timer, so lets stop it
         gobject.source_remove( self.watchTimeout )
         self.watchTimeout = None
@@ -422,7 +470,6 @@ class QueuedVirtualChannel( VirtualChannel ):
         #   reqstring, ok_cb, error_cb, timeout = request
         #   error_cb( reqstring.strip(), ( "cancel", timeout ) )
 
-    #@logged
     def _handleUnsolicitedResponse( self, response ):
         """
         Called from parser, when an unsolicited response has been parsed.
@@ -430,7 +477,6 @@ class QueuedVirtualChannel( VirtualChannel ):
         self.handleUnsolicitedResponse( response )
         return self.isWaitingForResponse() # parser needs to know the current status
 
-    #@logged
     def _handleResponseToRequest( self, response ):
         """
         Called from parser, when a response to a request has been parsed.
@@ -444,10 +490,9 @@ class QueuedVirtualChannel( VirtualChannel ):
         self.handleResponseToRequest( request, response )
         # relaunch
         if not self.watchReadyToSend:
-            self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend )
+            self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend, priority=PRIORITY_RTS )
         return self.isWaitingForResponse() # parser needs to know the current status
 
-    #@logged
     def _handleCommandTimeout( self ):
         """
         Called from mainloop, when a command does not get a response within a certain timeout.
@@ -459,6 +504,9 @@ class QueuedVirtualChannel( VirtualChannel ):
         self.watchTimeout = None
         self.serial.write( "\x1A" )
         self.handleCommandTimeout( self.q.get() )
+        # relaunch
+        if not self.watchReadyToSend:
+            self.watchReadyToSend = gobject.io_add_watch( self.serial.fd, gobject.IO_OUT, self._readyToSend, priority=PRIORITY_RTS )
         return False
 
 #=========================================================================#
@@ -490,10 +538,11 @@ class DelegateChannel( QueuedVirtualChannel ):
         """
         Set a delegate object to which all unsolicited responses are delegated first.
         """
-        assert self.delegate is None, "delegate already set"
+        if self.delegate is not None:
+            logger.warning( "delegate already set. Ignoring" )
+            return
         self.delegate = object
 
-    #@logged
     def _handleUnsolicitedResponse( self, response ):
         """
         Reimplemented for internal purposes.
@@ -505,7 +554,7 @@ class DelegateChannel( QueuedVirtualChannel ):
 
         data = response[0]
 
-        if not self.delegate:
+        if self.delegate is None:
             # no delegate installed, hand over to generic handler
             return self.handleUnsolicitedResponse( data )
 
@@ -516,8 +565,6 @@ class DelegateChannel( QueuedVirtualChannel ):
         command, values = data.split( ':', 1 )
 
         # convert unsolicited command to a method name
-        # FIXME this actually quite assumes certain structure for unsolicited responses,
-        # might think about making this more generic and/or override in AtCommandChannel
         command = command.replace( ' ', '_' ) # no spaces in Python identifiers
         methodname = "%s%s" % ( self.prefixmap[command[0]], command[1:] ) # no special characters
 
@@ -534,7 +581,7 @@ class DelegateChannel( QueuedVirtualChannel ):
                 else:
                     method( values.strip() )
             except Exception, e:
-                logger.exception( "unhandled exception in unsolicited response handler: %s" % e )
+                logger.exception( "(ignoring) unhandled exception in unsolicited response handler: %s" % e )
                 return False
 
         return True # unsolicited response handled OK
@@ -547,19 +594,29 @@ class AtCommandChannel( DelegateChannel ):
 
     Commands are prefixed according to v25ter. Multiline commands are handled.
     """
-    def enqueue( self, command, response_cb=None, error_cb=None, timeout=None ):
+
+    def enqueue( self, command, response_cb=None, error_cb=None, prefixes=None ):
         """
         Enqueue a single line or multiline command. Multiline commands have
         a '\r' (NOT '\r\n') embedded after the first line.
         """
         commands = command.split( '\r', 1 )
+
         if len( commands ) == 1:
-            QueuedVirtualChannel.enqueue( self, "AT%s\r\n" % command, response_cb, error_cb, timeout )
+            QueuedVirtualChannel.enqueue( self, "AT%s\r\n" % command, response_cb, error_cb, prefixes )
+
         elif len( commands ) == 2:
-            QueuedVirtualChannel.enqueue( self, "AT%s\r" % commands[0], None, None, None )
-            QueuedVirtualChannel.enqueue( self, "%s\x1A" % commands[1], response_cb, error_cb, timeout )
-        else:
-            assert False, "your python interpreter is broken"
+            QueuedVirtualChannel.enqueue( self, "AT%s\r" % commands[0], self.onMultilineCommandResponse, self.onMultilineCommandError, prefixes )
+            QueuedVirtualChannel.enqueue( self, "%s\x1A" % commands[1], response_cb, error_cb, prefixes )
+
+    def onMultilineCommandResponse( self, request, response ):
+        if response != []:
+            logger.warning( "multiline command got bogus response '%s' after first line. pushing ERROR to upper layer" )
+            self._handleResponseToRequest( "+EXT I: INTERNAL" )
+
+    def onMultilineCommandError( self, request, error ):
+        logger.error( "multiline command got error after first line. pushing ERROR to upper layer" )
+        self._handleResponseToRequest( "+EXT I: INTERNAL" )
 
     # you should not need to call this anymore
     enqueueRaw = QueuedVirtualChannel.enqueue
